#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 threshold;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 FragPosLightSpace;
} vert;


uniform bool lightB;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
layout(binding=0) uniform sampler2D diffuseTexture;
//layout(binding=1) uniform sampler2Dshadow shadowMap;
layout(binding=1) uniform sampler2D shadowMap;


uniform float gamma;


uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
	float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
	float currentDepth = projCoords.z;
    // calculate bias (based on depth map resolution and slope)
	vec3 normal = normalize(vert.normal_world);
	//vec3 lightDir = normalize(dirL.Position!!! - vert.position_world);
	//vec3 lightDir = dirL.direction;
	vec3 lightDir = normalize(dirL.direction - vert.position_world);
	//float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
	float bias = 0.001*tan(acos(dot(normal,lightDir))); 
	bias = clamp(bias, 0.0, 0.01); // clamped between 0 and 0.01
	 
	 // check whether current frag pos is in shadow
    // float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;
    // PCF
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
			if (currentDepth - bias > pcfDepth){
				shadow += 1.0;
			}
            //shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
    
    // keep the shadow at 0.0 when outside the far_plane region of the light's frustum.
    if(projCoords.z > 1.0)
        shadow = 0.0;
        
    return shadow;
}


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
float shadow = ShadowCalculation(vert.FragPosLightSpace); 
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return ((1.0 - shadow) * diffuseF * diffuseC * max(0, dot(n, l)) + (1.0 - shadow) *specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
		
		
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	

	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);

  
	
	 vec4 color2;

	 float intensity = dot(normalize(vert.FragPosLightSpace.rgb),vert.normal_world);

    if (intensity > 0.95)      color2 = vec4(1.0, 1.0, 1.0, 1.0);
    else if (intensity > 0.75) color2 = vec4(0.8, 0.8, 0.8, 1.0);
    else if (intensity > 0.50) color2 = vec4(0.6, 0.6, 0.6, 1.0);
    else if (intensity > 0.25) color2 = vec4(0.4, 0.4, 0.4, 1.0);
    else                       color2 = vec4(0.2, 0.2, 0.2, 1.0);

	if(lightB){
		threshold = vec4(texture(diffuseTexture, vert.uv).rgb, 1.0);
		}
	else{
		threshold = vec4(0.0, 0.0, 0.0, 1.0);
	}

	
	
	color = color *color2;
	color.rgb = pow(color.rgb, vec3(1.0/gamma));
	}

